#!/usr/bin/perl -w
#
# Main code for reLyX - the LaTeX to LyX translator
#
# reLyX is copyright (c) 1998 Amir Karger
# You are free to use and modify this code under the terms of
# the GNU General Public Licence version 2 or later.
#
# This code usually gets called by the reLyX wrapper executable
#
# $Id: reLyXmain.pl,v 1.10 1998/10/14 14:28:14 karger Exp $
#

require 5.002; # Perl 5.001 doesn't work. Perl 4 REALLY doesn't work.

# Standard Perl Library modules
use strict; # must define all variables, no barewords, no hard references

# Variables from other files, like Getopt::Std, OR vars that need to be global
# $opt_f etc. are command line options set by getopts
# dot_lyxdir is the name of the personal .lyx directory
#    @suffixlist is the list of possible suffixes for a tex file, which is used
# by File::Basename::fileparse
#    @File_List is the list of files to translate. Initially this is just
# the file input by the user, but \input or \include commands will add to it.
use vars qw($opt_c $opt_d $opt_f $opt_p $opt_r $opt_s $dot_lyxdir @File_List
            @suffixlist 
	   );
use Getopt::Std; # read in command-line options
use File::Basename; # &basename and &dirname

# Modules containing subroutines for reLyX
# Note that @INC must include the directory that these modules are in
use Text::TeX; # TeX parser package
use ReadCommands; # package to read LaTeX commands' syntax from a file
use MakePreamble; # package to split off LaTeX preamble & translate it to LyX
use CleanTeX; # package to clean TeX file for Lyxifying
use BasicLyX; # package to translate clean TeX to Basic LyX
use LastLyX; # package to print out LyX file once all translation is done


# Hack to allow running reLyX without the wrapper
if (!defined($lyxdir)) {$lyxdir = "/usr/local/share/lyx"}
if (!defined($lyxname)) {$lyxname = "lyx"}
# there's a use vars in the wrapper, so we only need these if running w/out it
use vars qw($lyxdir $lyxname);
 
# variables that a user might want to change
@suffixlist = '\.(ltx|latex|tex)'; # allowed suffixes for TeX file
my $LyXFormat = "2.15"; #What to print in \lyxformat command in .lyx file
my $syntaxname = "syntax.default"; # name of the default syntax file
$dot_lyxdir = $ENV{'HOME'} . "/.$lyxname"; # personal .lyx directory

##############################################################################
# MAIN
#

# Important variables!

# Print welcome message including version info
my $version_info = '$Date: 1998/10/14 14:28:14 $'; # RCS puts checkin date here
$version_info =~ s&.*?(\d+/\d+/\d+).*&$1&; # take out just the date info
warn "reLyX, the LaTeX to LyX translator. Revision date $version_info\n";

# Read command line
# Get Options: set $opt_f etc. based on command line options
getopts('c:dfpr:s:') or die "Illegal option! man reLyX for details\n";
die "-p option requires -c!\n" if defined $opt_p && !defined $opt_c;
die "no LaTeX file was input on the command line" unless @ARGV;
@File_List = @ARGV;
die "Only one input file allowed\n" if (@ARGV > 1 && !$opt_p);

# Read file(s) containing LaTeX commands and their syntax
# Read personal syntax.default, or system-wide if there isn't a personal one
# Then read other syntax files, given by the -s option
my $default_file = "$dot_lyxdir/reLyX/$syntaxname";
if (! -e $default_file) {
    $default_file = "$lyxdir/reLyX/$syntaxname";
    die "cannot find default syntax file $default_file" unless -e $default_file;
}
my @syntaxfiles = ($default_file);
push (@syntaxfiles, (split(/,/,$opt_s))) if defined $opt_s;
&ReadCommands::read_syntax_files(@syntaxfiles);

########### Main loop over files (include files will be added to @File_List)
my $count=0; #number of files we've done
my @deletelist = (); # files to delete when we're done (if no '-d' flag)
my $Doc_Class; # LaTeX documentclass
my $LyX_Preamble = "";   # LyX Preamble not including Latex preamble part
my $Latex_Preamble = "";
my $File;
while ($File = shift(@File_List)) {
    my $OutFileName;
    my $filenum = 0; #numbering for temporary files

    # Make sure the file is valid (main file must exist; others just warn)
    # PathBase is InFileName without .tex ending.For building output filenames
    my ($InFileName, $PathBase) = &test_file($File);
    if ($count) {
	next unless defined $InFileName;
    } else {
	die "\n" unless defined $InFileName; # test_file printed error already
    }
    print STDERR "($InFileName: ";

    # Do initial setup on first time through
    unless ($count) {

	if ($opt_p) { # it's a partial file
	    $Doc_Class = $opt_c;
	    $LyX_Preamble = $Latex_Preamble = ""; # no preamble nec.
	} else {
	    # Split the preamble off of the rest of the file

	    my $PreambleName = $PathBase . ".preamble";
	    die "\n$PreambleName exists\n" if (-e $PreambleName && !$opt_f);
	    $OutFileName = $PathBase . ".relyx" . ++$filenum;
	    push @deletelist, $OutFileName, $PreambleName;
	    &MakePreamble::split_preamble($InFileName,
	                            $PreambleName, $OutFileName);
	    $InFileName = $OutFileName;

	    # Now read and translate the LaTeX preamble into LyX
	    # Return document's class so we know which layout file(s) to read
	    # Also return LyX preamble in a string, for later
	    ($Doc_Class, $LyX_Preamble, $Latex_Preamble) =
		 &MakePreamble::translate_preamble($PreambleName, $LyXFormat);

	} # end partial file if

	# Read file(s) containing the valid LyX layouts for this documentclass
	# and their LaTeX command/environment equivalents
	&ReadCommands::read_layout_files($Doc_Class);
    }

    # Clean the TeX file (not including its preamble)
    $OutFileName = $PathBase . ".relyx" . ++$filenum;
    push @deletelist, $OutFileName;
    &CleanTeX::call_parser($InFileName, $OutFileName);

    # Now convert basic constructs in the cleaned TeX file to LyX constructs
    $InFileName = $OutFileName;
    $OutFileName = $PathBase . ".relyx" . ++$filenum;
    push @deletelist, $OutFileName;
    &BasicLyX::call_parser($InFileName, $OutFileName);

    # Finally, print out the actual LyX file including the preamble
    # For the *first* file, print out the LaTeX preamble too
    $InFileName = $OutFileName;
    $OutFileName = $PathBase . ".lyx";
    my $preamble = $count ? $LyX_Preamble : $LyX_Preamble . $Latex_Preamble;
    &LastLyX::last_lyx($InFileName, $OutFileName, $preamble);

    warn ")\n";

} continue {
    $count++;

} # END MAIN WHILE LOOP

# Cleanup
unless ($opt_d) {
    warn "Deleting temp files\n";
    unlink @deletelist;
}

my $Success = 1;
exit;

# If we "die", output a sad message
END{
    if (defined $Success) {
	warn "Finished successfully!\n";
    } else {
	warn "Exited due to fatal Error!\n";
    }
}

##################### SUBROUTINES  ###########################################
# Returns the input file name, as well as the 'pathbase', which
#    is the input file name (including path) without the '.tex' ending.
#    It will be used for building temporary and output file names
# Only allow certain suffixes
# Add .tex to filename if there's no suffix
# Return undef if there's an error
sub test_file {
    my $InFileName = shift;

    # Get file names, set up for making different temporary files
    # fileparse_set_fstype("MSDOS") for DOS support!
    my ($basename, $path, $suffix) = fileparse($InFileName, @suffixlist);
    $path .= '/' unless $path =~ /\/$/; # fix BUG in perl5.002 fileparse!

    # Try adding .tex to filename if you can't find the file the user input
    unless (-e $InFileName) {
        if (! $suffix) { # didn't have a valid suffix. Try adding one
	    if (-e "$InFileName.tex") {
		$suffix = ".tex";
	        $InFileName .= $suffix;
	    } else {
	 	warn "Cannot find input file $InFileName or $InFileName.tex\n";
		return (undef,undef);
	    }
	} else { # it had a valid suffix, but the file doesn't exist
	    warn "Cannot find input file $InFileName\n";
	    return (undef,undef);
	}
    }
    my $PathBase = $path . $basename; # all but suffix

    # Check for files that already exist
    my $lname = $PathBase . ".lyx";
    if (-e $lname && !$opt_f) {
	warn "LyX file $lname already exists\n";
	return (undef,undef);
    }

    return ($InFileName, $PathBase);
}


1; # return "true" to the wrapper
